namespace com.progsbase.no.inductive.libraries.PolynomialSolver_v0_1_9{

using static System.Math;

public class ComplexPolynomial{
	public cComplexNumber [] cs;
}
public class cComplexNumber{
	public double re;
	public double im;
}
public class cComplexNumberArrayReference{
	public cComplexNumber [] complexNumbers;
}
public class cComplexNumberReference{
	public cComplexNumber complexNumbers;
}
public class cPolarComplexNumber{
	public double r;
	public double phi;
}
public class BooleanArrayReference{
	public bool [] booleanArray;
}
public class BooleanReference{
	public bool booleanValue;
}
public class CharacterReference{
	public char characterValue;
}
public class NumberArrayReference{
	public double [] numberArray;
}
public class NumberReference{
	public double numberValue;
}
public class StringArrayReference{
	public StringReference [] stringArray;
}
public class StringReference{
	public char [] stringx;
}
public class LinkedListNodeStrings{
	public bool end;
	public char [] value;
	public LinkedListNodeStrings next;
}
public class LinkedListStrings{
	public LinkedListNodeStrings first;
	public LinkedListNodeStrings last;
}
public class LinkedListNodeNumbers{
	public LinkedListNodeNumbers next;
	public bool end;
	public double value;
}
public class LinkedListNumbers{
	public LinkedListNodeNumbers first;
	public LinkedListNodeNumbers last;
}
public class LinkedListCharacters{
	public LinkedListNodeCharacters first;
	public LinkedListNodeCharacters last;
}
public class LinkedListNodeCharacters{
	public bool end;
	public char value;
	public LinkedListNodeCharacters next;
}
public class DynamicArrayNumbers{
	public double [] array;
	public double length;
}
public class PolynomialSolver{
	public static bool FindRoots(double [] p, NumberArrayReference rootsReference){
		return DurandKernerMethod(p, 0.000001, 100d, rootsReference);
	}


	public static bool LaguerresMethodWithRepeatedDivision(double [] p, double maxIterations, double precision, double guess, NumberArrayReference rootsReference){
		double n, nr, xk;
		double [] x;
		double [] q, r, d;
		bool success;
		NumberReference xkReference;

		n = Degree(p);

		x = new double [(int)(n)];

		q = CreatePolynomial(n);
		r = CreatePolynomial(n);
		d = CreatePolynomial(n);

		success = true;
		xkReference = CreateNumberReference(0d);

		for(nr = 0d; nr < n && success; nr = nr + 1d){
			success = LaguerresMethod(p, guess, maxIterations, precision, xkReference);

			if(success){
				xk = xkReference.numberValue;
				x[(int)(nr)] = xk;

				Fill(d, 0d);
				d[0] = -xk;
				d[1] = 1d;
				Divide(q, r, p, d);
				Assign(p, q);
			}
		}

		delete(q);
		delete(r);
		delete(d);
        
		rootsReference.numberArray = x;

		return success;
	}


	public static bool LaguerresMethod(double [] p, double guess, double maxIterations, double precision, NumberReference rootReference){
		double k, a, G, H, xk, denom1, denom2, denom, t1, n;
		bool success;

		n = Degree(p);
		success = true;

		xk = guess;

		for(k = 0d; (k < maxIterations) && (Abs(Evaluate(p, xk)) >= precision) && success; k = k + 1d){
			G = EvaluateDerivative(p, xk, 1d)/Evaluate(p, xk);
			H = Pow(G, 2d) - EvaluateDerivative(p, xk, 2d)/Evaluate(p, xk);
			t1 = (n - 1d)*(n*H - Pow(G, 2d));
			if(t1 >= 0d){
				denom = Sqrt(t1);
				denom1 = G + denom;
				denom2 = G - denom;
				if(Abs(denom1) >= Abs(denom2)){
					denom = denom1;
				}else{
					denom = denom2;
				}
				a = n/denom;

				xk = xk - a;
			}else{
				success = false;
			}
		}

		if(k == maxIterations){
			success = false;
		}

		if(Abs(Evaluate(p, xk)) >= precision){
			success = false;
		}

		rootReference.numberValue = xk;

		return success;
	}


	public static bool DurandKernerMethod(double [] p, double precision, double maxIterations, NumberArrayReference rootsReference){
		bool success;
		double n, i, j, k, t1, t2, xn1, withinPrecision;
		double [] rs, rsPrev;

		n = p.Length - 1d;
		rs = new double [(int)(n)];
		rsPrev = new double [(int)(n)];

		for(i = 0d; i < n; i = i + 1d){
			rsPrev[(int)(i)] = i;
		}

		success = false;

		for(i = 0d; i < maxIterations && !success; i = i + 1d){
			for(j = 0d; j < n; j = j + 1d){
				xn1 = rsPrev[(int)(j)];

				t1 = Evaluate(p, xn1);
				t2 = 1d;
				for(k = 0d; k < n; k = k + 1d){
					if(k < j){
						t2 = t2*(xn1 - rs[(int)(k)]);
					}
					if(k > j){
						t2 = t2*(xn1 - rsPrev[(int)(k)]);
					}
				}
				t1 = t1/t2;
				rs[(int)(j)] = xn1 - t1;
			}
			withinPrecision = 0d;
			for(j = 0d; j < n; j = j + 1d){
				if(EpsilonCompare(rsPrev[(int)(j)], rs[(int)(j)], precision)){
					withinPrecision = withinPrecision + 1d;
				}
				rsPrev[(int)(j)] = rs[(int)(j)];
			}
			if(withinPrecision == n){
				success = true;
			}
		}

		rootsReference.numberArray = rs;

		return success;
	}


	public static bool FindRootsComplex(ComplexPolynomial p, cComplexNumberArrayReference rootsReference){
		return DurandKernerMethodComplex(p, 0.000001, 100d, rootsReference);
	}


	public static bool DurandKernerMethodComplex(ComplexPolynomial p, double precision, double maxIterations, cComplexNumberArrayReference rootsReference){
		bool success;
		double n, i, j, k, withinPrecision;
		cComplexNumber t1, t2, t3, xn1;
		cComplexNumber [] rs, rsPrev;

		n = p.cs.Length - 1d;
		rs = new cComplexNumber [(int)(n)];
		for(i = 0d; i < n; i = i + 1d){
			rs[(int)(i)] = cCreateComplexNumber(0d, 0d);
		}
		rsPrev = new cComplexNumber [(int)(n)];
		for(i = 0d; i < n; i = i + 1d){
			rsPrev[(int)(i)] = cCreateComplexNumber(0.4, 0.9);
			cPower(rsPrev[(int)(i)], i);
		}
		t2 = cCreateComplexNumber(0d, 0d);
		t3 = cCreateComplexNumber(0d, 0d);

		success = false;

		for(i = 0d; i < maxIterations && !success; i = i + 1d){
			for(j = 0d; j < n; j = j + 1d){
				xn1 = rsPrev[(int)(j)];

				t1 = EvaluateComplex(p, xn1);
				cAssignComplexByValues(t2, 1d, 0d);
				for(k = 0d; k < n; k = k + 1d){
					if(k < j){
						cAssignComplex(t3, xn1);
						cSub(t3, rs[(int)(k)]);
						cMul(t2, t3);
					}
					if(k > j){
						cAssignComplex(t3, xn1);
						cSub(t3, rsPrev[(int)(k)]);
						cMul(t2, t3);
					}
				}
				cDiv(t1, t2);
				cAssignComplex(rs[(int)(j)], xn1);
				cSub(rs[(int)(j)], t1);
			}
			withinPrecision = 0d;
			for(j = 0d; j < n; j = j + 1d){
				if(cEpsilonCompareComplex(rsPrev[(int)(j)], rs[(int)(j)], precision)){
					withinPrecision = withinPrecision + 1d;
				}
				cAssignComplex(rsPrev[(int)(j)], rs[(int)(j)]);
			}
			if(withinPrecision == n){
				success = true;
			}
		}

		rootsReference.complexNumbers = rs;

		return success;
	}


	public static char [] PolynomialToTextDirect(double [] p, char [] x){
		LinkedListCharacters ll;
		char [] str;
		double i, c, j;
		StringReference buffer;

		buffer = new StringReference();

		ll = CreateLinkedListCharacter();

		if(p.Length == 0d){
			LinkedListAddCharacter(ll, '0');
		}else{
			for(i = 0d; i < p.Length; i = i + 1d){
				c = p[(int)(i)];
				if(c < 0d){
					LinkedListAddCharacter(ll, '-');
				}else{
					LinkedListAddCharacter(ll, '+');
				}

				nCreateStringFromNumberWithCheck(Abs(c), 10d, buffer);
				LinkedListCharactersAddString(ll, buffer.stringx);
				delete(buffer.stringx);

				LinkedListCharactersAddString(ll, x);
				LinkedListAddCharacter(ll, '^');

				nCreateStringFromNumberWithCheck(i, 10d, buffer);
				LinkedListCharactersAddString(ll, buffer.stringx);
				delete(buffer.stringx);
			}
		}

		str = LinkedListCharactersToArray(ll);
		FreeLinkedListCharacter(ll);

		return str;
	}


	public static void LinkedListCharactersAddString(LinkedListCharacters ll, char [] str){
		double j;

		for(j = 0d; j < str.Length; j = j + 1d){
			LinkedListAddCharacter(ll, str[(int)(j)]);
		}
	}


	public static void GenerateCommonRenderSpecification(double [] p, BooleanArrayReference showCoefficient, StringReference sign, NumberArrayReference coefficient, BooleanArrayReference showPower, BooleanArrayReference showX){
		double i, zeros, c;
		bool setZero;

		setZero = false;

		if(p.Length == 0d){
			setZero = true;
		}else{
			zeros = 0d;
			for(i = 0d; i < p.Length; i = i + 1d){
				if(p[(int)(i)] == 0d){
					zeros = zeros + 1d;
				}
			}

			if(zeros == p.Length){
				setZero = true;
			}else{
				showCoefficient.booleanArray = new bool [(int)(p.Length)];
				sign.stringx = new char [(int)(p.Length)];
				coefficient.numberArray = new double [(int)(p.Length)];
				showPower.booleanArray = new bool [(int)(p.Length)];
				showX.booleanArray = new bool [(int)(p.Length)];

				for(i = 0d; i < p.Length; i = i + 1d){
					c = p[(int)(i)];

					if(c < 0d){
						sign.stringx[(int)(i)] = '-';
					}else{
						sign.stringx[(int)(i)] = '+';
					}
					coefficient.numberArray[(int)(i)] = Abs(p[(int)(i)]);
					if(c == 0d){
						showCoefficient.booleanArray[(int)(i)] = false;
					}else{
if(Abs(c) == 1d && i > 0d){
							showCoefficient.booleanArray[(int)(i)] = false;
						}else{
							showCoefficient.booleanArray[(int)(i)] = true;
						}

						if(i == 0d){
							showX.booleanArray[(int)(i)] = false;
							showPower.booleanArray[(int)(i)] = false;
						}else{
							showX.booleanArray[(int)(i)] = true;
							if(i == 1d){
								showPower.booleanArray[(int)(i)] = false;
							}else{
								showPower.booleanArray[(int)(i)] = true;
							}
						}
					}
				}
			}
		}

		if(setZero){
			showCoefficient.booleanArray = new bool [1];
			sign.stringx = new char [1];
			coefficient.numberArray = new double [1];
			showPower.booleanArray = new bool [1];
			showX.booleanArray = new bool [1];

			showCoefficient.booleanArray[0] = true;
			sign.stringx[0] = '+';
			coefficient.numberArray[0] = 0d;
			showPower.booleanArray[0] = true;
			showX.booleanArray[0] = false;
		}
	}


	public static char [] PolynomialToText(double [] p, char [] x){
		LinkedListCharacters ll;
		char [] str;
		double i, c;
		StringReference buffer;
		BooleanArrayReference showCoefficient, showPower, showX;
		StringReference sign;
		NumberArrayReference coefficient;
		bool hasPrinted;

		showCoefficient = CreateBooleanArrayReferenceLengthValue(0d, false);
		showPower = CreateBooleanArrayReferenceLengthValue(0d, false);
		showX = CreateBooleanArrayReferenceLengthValue(0d, false);
		sign = CreateStringReferenceLengthValue(0d, ' ');
		coefficient = CreateNumberArrayReferenceLengthValue(0d, 0d);

		GenerateCommonRenderSpecification(p, showCoefficient, sign, coefficient, showPower, showX);

		buffer = CreateStringReferenceLengthValue(0d, ' ');

		ll = CreateLinkedListCharacter();

		hasPrinted = false;
		for(i = 0d; i < showCoefficient.booleanArray.Length; i = i + 1d){
			c = p[(int)(i)];

			if(showCoefficient.booleanArray[(int)(i)] || showX.booleanArray[(int)(i)]){
				if(!hasPrinted && c >= 0d){
				}else{
					LinkedListAddCharacter(ll, sign.stringx[(int)(i)]);
				}

				if(showCoefficient.booleanArray[(int)(i)]){
					nCreateStringFromNumberWithCheck(coefficient.numberArray[(int)(i)], 10d, buffer);
					LinkedListCharactersAddString(ll, buffer.stringx);
					delete(buffer.stringx);
					hasPrinted = true;
				}

				if(showX.booleanArray[(int)(i)]){
					LinkedListCharactersAddString(ll, x);
					hasPrinted = true;
					if(showPower.booleanArray[(int)(i)]){
						LinkedListAddCharacter(ll, '^');
						nCreateStringFromNumberWithCheck(i, 10d, buffer);
						LinkedListCharactersAddString(ll, buffer.stringx);
						delete(buffer.stringx);
					}
				}
			}
		}

		str = LinkedListCharactersToArray(ll);
		FreeLinkedListCharacter(ll);

		return str;
	}


	public static char [] ComplexPolynomialToTextDirect(ComplexPolynomial p, char [] x){
		LinkedListCharacters ll;
		char [] str, number;
		double i;
		cComplexNumber c;
		StringReference buffer;

		buffer = new StringReference();

		ll = CreateLinkedListCharacter();

		if(p.cs.Length == 0d){
			LinkedListAddCharacter(ll, '0');
		}else{
			for(i = 0d; i < p.cs.Length; i = i + 1d){
				c = p.cs[(int)(i)];
				if(i > 0d){
					LinkedListAddCharacter(ll, '+');
				}

				number = cComplexToString(c);
				LinkedListAddCharacter(ll, '(');
				LinkedListCharactersAddString(ll, number);
				LinkedListAddCharacter(ll, ')');
				delete(number);

				LinkedListCharactersAddString(ll, x);
				LinkedListAddCharacter(ll, '^');

				nCreateStringFromNumberWithCheck(i, 10d, buffer);
				LinkedListCharactersAddString(ll, buffer.stringx);
				delete(buffer.stringx);
			}
		}

		str = LinkedListCharactersToArray(ll);
		FreeLinkedListCharacter(ll);

		return str;
	}


	public static void Add(double [] a, double [] b){
		double i, nr;

		nr = b.Length;

		for(i = 0d; i < nr; i = i + 1d){
			a[(int)(i)] = a[(int)(i)] + b[(int)(i)];
		}
	}


	public static void Subtract(double [] a, double [] b){
		double i, nr;

		nr = b.Length;

		for(i = 0d; i < nr; i = i + 1d){
			a[(int)(i)] = a[(int)(i)] - b[(int)(i)];
		}
	}


	public static void Multiply(double [] c, double [] a, double [] b){
		double k, n, m, i;
		double av, bv;

		n = Degree(a);
		m = Degree(b);

		Fill(c, 0d);

		for(i = 0d; i <= n + m; i = i + 1d){
			c[(int)(i)] = 0d;
			for(k = 0d; k <= i && k < a.Length && i - k < b.Length; k = k + 1d){
				av = a[(int)(k)];
				bv = b[(int)(i - k)];
				c[(int)(i)] = c[(int)(i)] + av*bv;
			}
		}
	}


	public static void Divide(double [] q, double [] r, double [] n, double [] d){
		double [] t, t1;
		double tcoff, tdegree, i;
		double deg;
		double rd, dd;

		Fill(q, 0d);
		Assign(r, n);
		deg = Degree(n);
		t = CreatePolynomial(deg);
		t1 = CreatePolynomial(deg);

		rd = Degree(r);
		dd = Degree(d);
		for(i = 0d; i < deg + 1d && !IsZero(r) && rd - i >= dd; i = i + 1d){
			Fill(t, 0d);
			tdegree = rd - i - dd;
			tcoff = r[(int)(rd - i)]/d[(int)(dd)];
			t[(int)(tdegree)] = tcoff;
			Add(q, t);
			Fill(t1, 0d);
			Multiply(t1, t, d);
			Subtract(r, t1);
		}

		delete(t);
		delete(t1);
	}


	public static bool IsZero(double [] a){
		double i;
		bool itIsZero;

		itIsZero = true;

		for(i = 0d; i < a.Length; i = i + 1d){
			if(a[(int)(i)] != 0d){
				itIsZero = false;
			}
		}

		return itIsZero;
	}


	public static void Assign(double [] a, double [] b){
		double i, nr;

		nr = b.Length;

		for(i = 0d; i < nr; i = i + 1d){
			a[(int)(i)] = b[(int)(i)];
		}
	}


	public static double [] CreatePolynomial(double deg){
		double [] p;

		p = new double [(int)(deg + 1d)];

		Fill(p, 0d);

		return p;
	}


	public static void Fill(double [] p, double value){
		double i;

		for(i = 0d; i < p.Length; i = i + 1d){
			p[(int)(i)] = value;
		}
	}


	public static double Degree(double [] A){
		double i;
		double deg;
		bool done;

		done = false;
		deg = 0d;
		for(i = A.Length - 1d; i >= 0d && !done; i = i - 1d){
			if(A[(int)(i)] != 0d){
				deg = i;
				done = true;
			}
		}

		return deg;
	}


	public static double Lead(double [] A){
		double deg;

		deg = Degree(A);

		return A[(int)(deg)];
	}


	public static double Evaluate(double [] A, double x){
		return EvaluateWithHornersMethod(A, x);
	}


	public static double EvaluateWithHornersMethod(double [] A, double x){
		double r, i;

		r = 0d;

		for(i = A.Length - 1d; i >= 0d; i = i - 1d){
			r = r*x;
			r = A[(int)(i)] + r;
		}

		return r;
	}


	public static double EvaluateWithPowers(double [] A, double x){
		double r, i;

		r = 0d;

		for(i = 0d; i < A.Length; i = i + 1d){
			r = r + A[(int)(i)]*Pow(x, i);
		}

		return r;
	}


	public static double EvaluateDerivative(double [] A, double x, double n){
		double r, i, v;

		r = 0d;

		for(i = 0d; i < A.Length; i = i + 1d){
			if(i - n >= 0d){
				/*v = A[(int) i] * mFactorial(n) * mCombinations(i, n) * pow(x, i - n);*/
				v = A[(int)(i)]*Permutations(i, n)*Pow(x, i - n);
				r = r + v;
			}
		}

		return r;
	}


	public static void Derivative(double [] A){
		double i, degree;

		degree = 0d;
		for(i = 1d; i < A.Length; i = i + 1d){
			degree = degree + 1d;
			A[(int)(i - 1d)] = degree*A[(int)(i)];
		}

		A[(int)(A.Length - 1d)] = 0d;
	}


	public static void AddComplex(ComplexPolynomial a, ComplexPolynomial b){
		double i, nr;

		nr = a.cs.Length;

		for(i = 0d; i < nr; i = i + 1d){
			cAdd(a.cs[(int)(i)], b.cs[(int)(i)]);
		}
	}


	public static void SubtractComplex(ComplexPolynomial a, ComplexPolynomial b){
		double i, nr;

		nr = a.cs.Length;

		for(i = 0d; i < nr; i = i + 1d){
			cSub(a.cs[(int)(i)], b.cs[(int)(i)]);
		}
	}


	public static bool IsZeroComplex(ComplexPolynomial a){
		double i;
		bool itIsZero;

		itIsZero = true;

		for(i = 0d; i < a.cs.Length; i = i + 1d){
			if(a.cs[(int)(i)].re != 0d && a.cs[(int)(i)].im != 0d){
				itIsZero = false;
			}
		}

		return itIsZero;
	}


	public static void AssignComplex(ComplexPolynomial a, ComplexPolynomial b){
		double i, nr;

		nr = b.cs.Length;

		for(i = 0d; i < nr; i = i + 1d){
			cAssignComplex(a.cs[(int)(i)], b.cs[(int)(i)]);
		}
	}


	public static ComplexPolynomial CreateComplexPolynomial(double deg){
		ComplexPolynomial p;
		double i;

		p = new ComplexPolynomial();
		p.cs = new cComplexNumber [(int)(deg + 1d)];

		for(i = 0d; i < deg + 1d; i = i + 1d){
			p.cs[(int)(i)] = new cComplexNumber();
		}

		FillComplex(p, 0d, 0d);

		return p;
	}


	public static void FillComplex(ComplexPolynomial p, double re, double im){
		double i;
		cComplexNumber c;

		c = cCreateComplexNumber(re, im);

		for(i = 0d; i < p.cs.Length; i = i + 1d){
			cAssignComplex(p.cs[(int)(i)], c);
		}

		delete(c);
	}


	public static double DegreeComplex(ComplexPolynomial A){
		double i;
		double deg;
		bool done;

		done = false;
		deg = 0d;
		for(i = A.cs.Length - 1d; i >= 0d && !done; i = i - 1d){
			if(A.cs[(int)(i)].re != 0d && A.cs[(int)(i)].im != 0d){
				deg = i;
				done = true;
			}
		}

		return deg;
	}


	public static cComplexNumber LeadComplex(ComplexPolynomial A){
		double deg;

		deg = DegreeComplex(A);

		return A.cs[(int)(deg)];
	}


	public static cComplexNumber EvaluateComplex(ComplexPolynomial A, cComplexNumber x){
		double i;
		cComplexNumber r, t;

		r = cCreateComplexNumber(0d, 0d);
		t = cCreateComplexNumber(0d, 0d);

		for(i = 0d; i < A.cs.Length; i = i + 1d){
			cAssignComplex(t, x);
			cPower(t, i);
			cMul(t, A.cs[(int)(i)]);
			cAdd(r, t);
		}

		return r;
	}


	public static double TotalNumberOfRoots(double [] p){
		return Degree(p);
	}


	public static double Negate(double x){
		return -x;
	}


	public static double Positive(double x){
		return +x;
	}


	public static double Factorial(double x){
		double i, f;

		f = 1d;

		for(i = 2d; i <= x; i = i + 1d){
			f = f*i;
		}

		return f;
	}


	public static double Roundx(double x){
		return Floor(x + 0.5);
	}


	public static double BankersRound(double x){
		double r;

		if(Absolute(x - Truncate(x)) == 0.5){
			if(!DivisibleBy(Roundx(x), 2d)){
				r = Roundx(x) - 1d;
			}else{
				r = Roundx(x);
			}
		}else{
			r = Roundx(x);
		}

		return r;
	}


	public static double Ceil(double x){
		return Ceiling(x);
	}


	public static double Floorx(double x){
		return Floor(x);
	}


	public static double Truncate(double x){
		double t;

		if(x >= 0d){
			t = Floor(x);
		}else{
			t = Ceiling(x);
		}

		return t;
	}


	public static double Absolute(double x){
		return Abs(x);
	}


	public static double Logarithm(double x){
		return Log10(x);
	}


	public static double NaturalLogarithm(double x){
		return Log(x);
	}


	public static double Sinx(double x){
		return Sin(x);
	}


	public static double Cosx(double x){
		return Cos(x);
	}


	public static double Tanx(double x){
		return Tan(x);
	}


	public static double Asinx(double x){
		return Asin(x);
	}


	public static double Acosx(double x){
		return Acos(x);
	}


	public static double Atanx(double x){
		return Atan(x);
	}


	public static double Atan2(double y, double x){
		double a;

		/* Atan2 is an invalid operation when x = 0 and y = 0, but this method does not return errors.*/
		a = 0d;

		if(x > 0d){
			a = Atanx(y/x);
		}else if(x < 0d && y >= 0d){
			a = Atanx(y/x) + PI;
		}else if(x < 0d && y < 0d){
			a = Atanx(y/x) - PI;
		}else if(x == 0d && y > 0d){
			a = PI/2d;
		}else if(x == 0d && y < 0d){
			a = -PI/2d;
		}

		return a;
	}


	public static double Squareroot(double x){
		return Sqrt(x);
	}


	public static double Expx(double x){
		return Exp(x);
	}


	public static bool DivisibleBy(double a, double b){
		return ((a%b) == 0d);
	}


	public static double Combinations(double n, double k){
		double i, j, c;

		c = 1d;
		j = 1d;
		i = n - k + 1d;

		for(; i <= n; ){
			c = c*i;
			c = c/j;

			i = i + 1d;
			j = j + 1d;
		}

		return c;
	}


	public static double Permutations(double n, double k){
		double i, c;

		c = 1d;

		for(i = n - k + 1d; i <= n; i = i + 1d){
			c = c*i;
		}

		return c;
	}


	public static bool EpsilonCompare(double a, double b, double epsilon){
		return Abs(a - b) < epsilon;
	}


	public static double GreatestCommonDivisor(double a, double b){
		double t;

		for(; b != 0d; ){
			t = b;
			b = a%b;
			a = t;
		}

		return a;
	}


	public static double GCDWithSubtraction(double a, double b){
		double g;

		if(a == 0d){
			g = b;
		}else{
			for(; b != 0d; ){
				if(a > b){
					a = a - b;
				}else{
					b = b - a;
				}
			}

			g = a;
		}

		return g;
	}


	public static bool IsInteger(double a){
		return (a - Floor(a)) == 0d;
	}


	public static bool GreatestCommonDivisorWithCheck(double a, double b, NumberReference gcdReference){
		bool success;
		double gcd;

		if(IsInteger(a) && IsInteger(b)){
			gcd = GreatestCommonDivisor(a, b);
			gcdReference.numberValue = gcd;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static double LeastCommonMultiple(double a, double b){
		double lcm;

		if(a > 0d && b > 0d){
			lcm = Abs(a*b)/GreatestCommonDivisor(a, b);
		}else{
			lcm = 0d;
		}

		return lcm;
	}


	public static double Sign(double a){
		double s;

		if(a > 0d){
			s = 1d;
		}else if(a < 0d){
			s = -1d;
		}else{
			s = 0d;
		}

		return s;
	}


	public static double Maxx(double a, double b){
		return Max(a, b);
	}


	public static double Minx(double a, double b){
		return Min(a, b);
	}


	public static double Power(double a, double b){
		return Pow(a, b);
	}


	public static double Gamma(double x){
		return LanczosApproximation(x);
	}


	public static double LogGamma(double x){
		return Log(Gamma(x));
	}


	public static double LanczosApproximation(double z){
		double [] p;
		double i, y, t, x;

		p = new double [8];
		p[0] = 676.5203681218851;
		p[1] = -1259.1392167224028;
		p[2] = 771.32342877765313;
		p[3] = -176.61502916214059;
		p[4] = 12.507343278686905;
		p[5] = -0.13857109526572012;
		p[6] = 9.9843695780195716e-6;
		p[7] = 1.5056327351493116e-7;

		if(z < 0.5){
			y = PI/(Sin(PI*z)*LanczosApproximation(1d - z));
		}else{
			z = z - 1d;
			x = 0.99999999999980993;
			for(i = 0d; i < p.Length; i = i + 1d){
				x = x + p[(int)(i)]/(z + i + 1d);
			}
			t = z + p.Length - 0.5;
			y = Sqrt(2d*PI)*Pow(t, z + 0.5)*Exp(-t)*x;
		}

		return y;
	}


	public static double Beta(double x, double y){
		return Gamma(x)*Gamma(y)/Gamma(x + y);
	}


	public static double Sinh(double x){
		return (Exp(x) - Exp(-x))/2d;
	}


	public static double Cosh(double x){
		return (Exp(x) + Exp(-x))/2d;
	}


	public static double Tanh(double x){
		return Sinh(x)/Cosh(x);
	}


	public static double Cot(double x){
		return 1d/Tan(x);
	}


	public static double Sec(double x){
		return 1d/Cos(x);
	}


	public static double Csc(double x){
		return 1d/Sin(x);
	}


	public static double Coth(double x){
		return Cosh(x)/Sinh(x);
	}


	public static double Sech(double x){
		return 1d/Cosh(x);
	}


	public static double Csch(double x){
		return 1d/Sinh(x);
	}


	public static double Error(double x){
		double y, t, tau, c1, c2, c3, c4, c5, c6, c7, c8, c9, c10;

		if(x == 0d){
			y = 0d;
		}else if(x < 0d){
			y = -Error(-x);
		}else{
			c1 = -1.26551223;
			c2 = +1.00002368;
			c3 = +0.37409196;
			c4 = +0.09678418;
			c5 = -0.18628806;
			c6 = +0.27886807;
			c7 = -1.13520398;
			c8 = +1.48851587;
			c9 = -0.82215223;
			c10 = +0.17087277;

			t = 1d/(1d + 0.5*Abs(x));

			tau = t*Exp(-Pow(x, 2d) + c1 + t*(c2 + t*(c3 + t*(c4 + t*(c5 + t*(c6 + t*(c7 + t*(c8 + t*(c9 + t*c10)))))))));

			y = 1d - tau;
		}

		return y;
	}


	public static double ErrorInverse(double x){
		double y, a, t;

		a = (8d*(PI - 3d))/(3d*PI*(4d - PI));

		t = 2d/(PI*a) + Log(1d - Pow(x, 2d))/2d;
		y = Sign(x)*Sqrt(Sqrt(Pow(t, 2d) - Log(1d - Pow(x, 2d))/a) - t);

		return y;
	}


	public static double FallingFactorial(double x, double n){
		double k, y;

		y = 1d;

		for(k = 0d; k <= n - 1d; k = k + 1d){
			y = y*(x - k);
		}

		return y;
	}


	public static double RisingFactorial(double x, double n){
		double k, y;

		y = 1d;

		for(k = 0d; k <= n - 1d; k = k + 1d){
			y = y*(x + k);
		}

		return y;
	}


	public static double Hypergeometric(double a, double b, double c, double z, double maxIterations, double precision){
		double y;

		if(Abs(z) >= 0.5){
			y = Pow(1d - z, -a)*HypergeometricDirect(a, c - b, c, z/(z - 1d), maxIterations, precision);
		}else{
			y = HypergeometricDirect(a, b, c, z, maxIterations, precision);
		}

		return y;
	}


	public static double HypergeometricDirect(double a, double b, double c, double z, double maxIterations, double precision){
		double y, yp, n;
		bool done;

		y = 0d;
		done = false;

		for(n = 0d; n < maxIterations && !done; n = n + 1d){
			yp = RisingFactorial(a, n)*RisingFactorial(b, n)/RisingFactorial(c, n)*Pow(z, n)/Factorial(n);
			if(Abs(yp) < precision){
				done = true;
			}
			y = y + yp;
		}

		return y;
	}


	public static double BernouilliNumber(double n){
		return AkiyamaTanigawaAlgorithm(n);
	}


	public static double AkiyamaTanigawaAlgorithm(double n){
		double m, j, B;
		double [] A;

		A = new double [(int)(n + 1d)];

		for(m = 0d; m <= n; m = m + 1d){
			A[(int)(m)] = 1d/(m + 1d);
			for(j = m; j >= 1d; j = j - 1d){
				A[(int)(j - 1d)] = j*(A[(int)(j - 1d)] - A[(int)(j)]);
			}
		}

		B = A[0];

		delete(A);

		return B;
	}


	public static cComplexNumber cCreateComplexNumber(double re, double im){
		cComplexNumber z;

		z = new cComplexNumber();
		z.re = re;
		z.im = im;

		return z;
	}


	public static cPolarComplexNumber cCreatePolarComplexNumber(double r, double phi){
		cPolarComplexNumber p;

		p = new cPolarComplexNumber();
		p.r = r;
		p.phi = phi;

		return p;
	}


	public static void cAdd(cComplexNumber z1, cComplexNumber z2){
		double a, b, c, d;

		a = z1.re;
		b = z1.im;
		c = z2.re;
		d = z2.im;

		z1.re = a + c;
		z1.im = b + d;
	}


	public static cComplexNumber cAddToNew(cComplexNumber z1, cComplexNumber z2){
		cComplexNumber x;
		double a, b, c, d;

		a = z1.re;
		b = z1.im;
		c = z2.re;
		d = z2.im;

		x = new cComplexNumber();

		x.re = a + c;
		x.im = b + d;

		return x;
	}


	public static void cSub(cComplexNumber z1, cComplexNumber z2){
		double a, b, c, d;

		a = z1.re;
		b = z1.im;
		c = z2.re;
		d = z2.im;

		z1.re = a - c;
		z1.im = b - d;
	}


	public static cComplexNumber cSubToNew(cComplexNumber z1, cComplexNumber z2){
		cComplexNumber x;
		double a, b, c, d;

		a = z1.re;
		b = z1.im;
		c = z2.re;
		d = z2.im;

		x = new cComplexNumber();

		x.re = a - c;
		x.im = b - d;

		return x;
	}


	public static void cMul(cComplexNumber z1, cComplexNumber z2){
		double a, b, c, d;

		a = z1.re;
		b = z1.im;
		c = z2.re;
		d = z2.im;

		z1.re = a*c - b*d;
		z1.im = b*c + a*d;
	}


	public static cComplexNumber cMulToNew(cComplexNumber z1, cComplexNumber z2){
		cComplexNumber x;
		double a, b, c, d;

		a = z1.re;
		b = z1.im;
		c = z2.re;
		d = z2.im;

		x = new cComplexNumber();

		x.re = a*c - b*d;
		x.im = b*c + a*d;

		return x;
	}


	public static void cDiv(cComplexNumber z1, cComplexNumber z2){
		double a, b, c, d;

		a = z1.re;
		b = z1.im;
		c = z2.re;
		d = z2.im;

		z1.re = (a*c + b*d)/(Pow(c, 2d) + Pow(d, 2d));
		z1.im = (b*c - a*d)/(Pow(c, 2d) + Pow(d, 2d));
	}


	public static cComplexNumber cDivToNew(cComplexNumber z1, cComplexNumber z2){
		cComplexNumber x;
		double a, b, c, d;

		a = z1.re;
		b = z1.im;
		c = z2.re;
		d = z2.im;

		x = new cComplexNumber();

		x.re = (a*c + b*d)/(Pow(c, 2d) + Pow(d, 2d));
		x.im = (b*c - a*d)/(Pow(c, 2d) + Pow(d, 2d));

		return x;
	}


	public static void cConjugate(cComplexNumber z){
		z.im = -z.im;
	}


	public static cComplexNumber cConjugateToNew(cComplexNumber z){
		cComplexNumber x;

		x = new cComplexNumber();

		x.re = z.re;
		x.im = -z.im;

		return x;
	}


	public static double cAbs(cComplexNumber z){
		double x;

		x = Sqrt(Pow(z.re, 2d) + Pow(z.im, 2d));

		return x;
	}


	public static double cArg(cComplexNumber z){
		double x;

		x = Atan2(z.im, z.re);

		return x;
	}


	public static cPolarComplexNumber cCreatePolarFromComplexNumber(cComplexNumber z){
		cPolarComplexNumber x;

		x = new cPolarComplexNumber();

		x.r = cAbs(z);
		x.phi = cArg(z);

		return x;
	}


	public static cComplexNumber cCreateComplexFromPolar(cPolarComplexNumber p){
		cComplexNumber z;

		z = new cComplexNumber();

		z.re = p.r*Cos(p.phi);
		z.im = p.r*Sin(p.phi);

		return z;
	}


	public static double cRe(cComplexNumber z){
		return z.re;
	}


	public static double cIm(cComplexNumber z){
		return z.im;
	}


	public static void cAddPolar(cPolarComplexNumber p1, cPolarComplexNumber p2){
		cPolarComplexNumber x;
		cComplexNumber z1, z2;

		z1 = cCreateComplexFromPolar(p1);
		z2 = cCreateComplexFromPolar(p2);

		cAdd(z1, z2);

		x = cCreatePolarFromComplexNumber(z1);

		p1.r = x.r;
		p1.phi = x.phi;

		delete(z1);
		delete(z2);
		delete(x);
	}


	public static cPolarComplexNumber cAddPolarToNew(cPolarComplexNumber p1, cPolarComplexNumber p2){
		cPolarComplexNumber x;
		cComplexNumber z1, z2;

		z1 = cCreateComplexFromPolar(p1);
		z2 = cCreateComplexFromPolar(p2);

		cAdd(z1, z2);

		x = cCreatePolarFromComplexNumber(z1);

		delete(z1);
		delete(z2);

		return x;
	}


	public static void cSubPolar(cPolarComplexNumber p1, cPolarComplexNumber p2){
		cPolarComplexNumber x;
		cComplexNumber z1, z2;

		z1 = cCreateComplexFromPolar(p1);
		z2 = cCreateComplexFromPolar(p2);

		cSub(z1, z2);

		x = cCreatePolarFromComplexNumber(z1);

		p1.r = x.r;
		p1.phi = x.phi;

		delete(z1);
		delete(z2);
		delete(x);
	}


	public static cPolarComplexNumber cSubPolarToNew(cPolarComplexNumber p1, cPolarComplexNumber p2){
		cPolarComplexNumber x;
		cComplexNumber z1, z2;

		z1 = cCreateComplexFromPolar(p1);
		z2 = cCreateComplexFromPolar(p2);

		cSub(z1, z2);

		x = cCreatePolarFromComplexNumber(z1);

		delete(z1);
		delete(z2);

		return x;
	}


	public static void cMulPolar(cPolarComplexNumber p1, cPolarComplexNumber p2){
		double r1, r2, phi1, phi2;

		r1 = p1.r;
		r2 = p2.r;
		phi1 = p1.phi;
		phi2 = p2.phi;

		p1.r = r1*r2;
		p1.phi = phi1 + phi2;
	}


	public static cPolarComplexNumber cMulPolarToNew(cPolarComplexNumber p1, cPolarComplexNumber p2){
		cPolarComplexNumber x;
		double r1, r2, phi1, phi2;

		r1 = p1.r;
		r2 = p2.r;
		phi1 = p1.phi;
		phi2 = p2.phi;

		x = new cPolarComplexNumber();

		x.r = r1*r2;
		x.phi = phi1 + phi2;

		return x;
	}


	public static void cDivPolar(cPolarComplexNumber p1, cPolarComplexNumber p2){
		double r1, r2, phi1, phi2;

		r1 = p1.r;
		r2 = p2.r;
		phi1 = p1.phi;
		phi2 = p2.phi;

		p1.r = r1/r2;
		p1.phi = phi1 - phi2;
	}


	public static cPolarComplexNumber cDivPolarToNew(cPolarComplexNumber p1, cPolarComplexNumber p2){
		cPolarComplexNumber x;
		double r1, r2, phi1, phi2;

		r1 = p1.r;
		r2 = p2.r;
		phi1 = p1.phi;
		phi2 = p2.phi;

		x = new cPolarComplexNumber();

		x.r = r1/r2;
		x.phi = phi1 - phi2;

		return x;
	}


	public static void cSquareRoot(cComplexNumber z){
		double a, b, m;

		a = z.re;
		b = z.im;

		m = Sqrt(Pow(a, 2d) + Pow(b, 2d));

		z.re = Sqrt((m + a)/2d);
		z.im = Sign(b)*Sqrt((m - a)/2d);
	}


	public static void cPowerPolar(cPolarComplexNumber p, double n){
		p.r = Pow(p.r, n);
		p.phi = p.phi*n;
	}


	public static cComplexNumber cPowerToNew(cComplexNumber z, double n){
		cPolarComplexNumber p;
		cComplexNumber zp;

		p = cCreatePolarFromComplexNumber(z);
		cPowerPolar(p, n);
		zp = cCreateComplexFromPolar(p);

		delete(p);

		return zp;
	}


	public static void cPower(cComplexNumber z, double n){
		cComplexNumber zp;

		zp = cPowerToNew(z, n);
		z.re = zp.re;
		z.im = zp.im;

		delete(zp);
	}


	public static void cNegate(cComplexNumber z){
		z.re = Negate(z.re);
		z.im = Negate(z.im);
	}


	public static void cAssignComplexByValues(cComplexNumber s, double re, double im){
		s.re = re;
		s.im = im;
	}


	public static void cAssignComplex(cComplexNumber a, cComplexNumber b){
		a.re = b.re;
		a.im = b.im;
	}


	public static bool cEpsilonCompareComplex(cComplexNumber a, cComplexNumber b, double epsilon){
		return EpsilonCompare(a.re, b.re, epsilon) && EpsilonCompare(a.im, b.im, epsilon);
	}


	public static void cExpComplex(cComplexNumber x){
		double re, im;

		re = Exp(x.re)*Cos(x.im);
		im = Exp(x.re)*Sin(x.im);
		x.re = re;
		x.im = im;
	}


	public static void cSineComplex(cComplexNumber x){
		double re, im;

		re = Sin(x.re)*Cosh(x.im);
		im = Cos(x.re)*Sinh(x.im);
		x.re = re;
		x.im = im;
	}


	public static void cCosineComplex(cComplexNumber x){
		double re, im;

		re = Cos(x.re)*Cosh(x.im);
		im = Sin(x.re)*Sinh(x.im);
		x.re = re;
		x.im = im;
	}


	public static char [] cComplexToString(cComplexNumber a){
		char [] str, number;
		LinkedListCharacters ll;
		double i;

		ll = CreateLinkedListCharacter();

		number = nCreateStringDecimalFromNumber(a.re);

		for(i = 0d; i < number.Length; i = i + 1d){
			LinkedListAddCharacter(ll, number[(int)(i)]);
		}

		delete(number);

		if(a.im < 0d){
			LinkedListAddCharacter(ll, '-');
			number = nCreateStringDecimalFromNumber(-a.im);
		}else{
			LinkedListAddCharacter(ll, '+');
			number = nCreateStringDecimalFromNumber(a.im);
		}

		for(i = 0d; i < number.Length; i = i + 1d){
			LinkedListAddCharacter(ll, number[(int)(i)]);
		}

		delete(number);

		LinkedListAddCharacter(ll, 'i');

		str = LinkedListCharactersToArray(ll);
		FreeLinkedListCharacter(ll);

		return str;
	}


	public static BooleanReference CreateBooleanReference(bool value){
		BooleanReference refx;

		refx = new BooleanReference();
		refx.booleanValue = value;

		return refx;
	}


	public static BooleanArrayReference CreateBooleanArrayReference(bool [] value){
		BooleanArrayReference refx;

		refx = new BooleanArrayReference();
		refx.booleanArray = value;

		return refx;
	}


	public static BooleanArrayReference CreateBooleanArrayReferenceLengthValue(double length, bool value){
		BooleanArrayReference refx;
		double i;

		refx = new BooleanArrayReference();
		refx.booleanArray = new bool [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.booleanArray[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeBooleanArrayReference(BooleanArrayReference booleanArrayReference){
		delete(booleanArrayReference.booleanArray);
		delete(booleanArrayReference);
	}


	public static CharacterReference CreateCharacterReference(char value){
		CharacterReference refx;

		refx = new CharacterReference();
		refx.characterValue = value;

		return refx;
	}


	public static NumberReference CreateNumberReference(double value){
		NumberReference refx;

		refx = new NumberReference();
		refx.numberValue = value;

		return refx;
	}


	public static NumberArrayReference CreateNumberArrayReference(double [] value){
		NumberArrayReference refx;

		refx = new NumberArrayReference();
		refx.numberArray = value;

		return refx;
	}


	public static NumberArrayReference CreateNumberArrayReferenceLengthValue(double length, double value){
		NumberArrayReference refx;
		double i;

		refx = new NumberArrayReference();
		refx.numberArray = new double [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.numberArray[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeNumberArrayReference(NumberArrayReference numberArrayReference){
		delete(numberArrayReference.numberArray);
		delete(numberArrayReference);
	}


	public static StringReference CreateStringReference(char [] value){
		StringReference refx;

		refx = new StringReference();
		refx.stringx = value;

		return refx;
	}


	public static StringReference CreateStringReferenceLengthValue(double length, char value){
		StringReference refx;
		double i;

		refx = new StringReference();
		refx.stringx = new char [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.stringx[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeStringReference(StringReference stringReference){
		delete(stringReference.stringx);
		delete(stringReference);
	}


	public static StringArrayReference CreateStringArrayReference(StringReference [] strings){
		StringArrayReference refx;

		refx = new StringArrayReference();
		refx.stringArray = strings;

		return refx;
	}


	public static StringArrayReference CreateStringArrayReferenceLengthValue(double length, char [] value){
		StringArrayReference refx;
		double i;

		refx = new StringArrayReference();
		refx.stringArray = new StringReference [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.stringArray[(int)(i)] = CreateStringReference(value);
		}

		return refx;
	}


	public static void FreeStringArrayReference(StringArrayReference stringArrayReference){
		double i;

		for(i = 0d; i < stringArrayReference.stringArray.Length; i = i + 1d){
			delete(stringArrayReference.stringArray[(int)(i)]);
		}
		delete(stringArrayReference.stringArray);
		delete(stringArrayReference);
	}


	public static void AssertFalse(bool b, NumberReference failures){
		if(b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertTrue(bool b, NumberReference failures){
		if(!b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertEquals(double a, double b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertBooleansEqual(bool a, bool b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertCharactersEqual(char a, char b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertStringEquals(char [] a, char [] b, NumberReference failures){
		if(!StringsEqual(a, b)){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertNumberArraysEqual(double [] a, double [] b, NumberReference failures){
		double i;

		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertEquals(a[(int)(i)], b[(int)(i)], failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertBooleanArraysEqual(bool [] a, bool [] b, NumberReference failures){
		double i;

		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertBooleansEqual(a[(int)(i)], b[(int)(i)], failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertStringArraysEqual(StringReference [] a, StringReference [] b, NumberReference failures){
		double i;

		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertStringEquals(a[(int)(i)].stringx, b[(int)(i)].stringx, failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static char [] nCreateStringScientificNotationDecimalFromNumber(double decimalx){
		StringReference mantissaReference, exponentReference;
		double multiplier, inc;
		double exponent;
		bool done, isPositive;
		char [] result;

		mantissaReference = new StringReference();
		exponentReference = new StringReference();
		result = new char [0];
		done = false;
		exponent = 0d;

		if(decimalx < 0d){
			isPositive = false;
			decimalx = -decimalx;
		}else{
			isPositive = true;
		}

		if(decimalx == 0d){
			done = true;
		}

		if(!done){
			multiplier = 0d;
			inc = 0d;

			if(decimalx < 1d){
				multiplier = 10d;
				inc = -1d;
			}else if(decimalx >= 10d){
				multiplier = 0.1;
				inc = 1d;
			}else{
				done = true;
			}

			if(!done){
				for(; decimalx >= 10d || decimalx < 1d; ){
					decimalx = decimalx*multiplier;
					exponent = exponent + inc;
				}
			}
		}

		nCreateStringFromNumberWithCheck(decimalx, 10d, mantissaReference);

		nCreateStringFromNumberWithCheck(exponent, 10d, exponentReference);

		if(!isPositive){
			result = AppendString(result, "-".ToCharArray());
		}

		result = AppendString(result, mantissaReference.stringx);
		result = AppendString(result, "e".ToCharArray());
		result = AppendString(result, exponentReference.stringx);

		return result;
	}


	public static char [] nCreateStringDecimalFromNumber(double decimalx){
		StringReference stringReference;

		stringReference = new StringReference();

		/* This will succeed because base = 10.*/
		nCreateStringFromNumberWithCheck(decimalx, 10d, stringReference);

		return stringReference.stringx;
	}


	public static bool nCreateStringFromNumberWithCheck(double decimalx, double basex, StringReference stringReference){
		char [] stringx;
		double maximumDigits;
		double digitPosition;
		bool hasPrintedPoint, isPositive;
		double i, d;
		bool success;
		CharacterReference characterReference;
		char c;

		isPositive = true;

		if(decimalx < 0d){
			isPositive = false;
			decimalx = -decimalx;
		}

		if(decimalx == 0d){
			stringReference.stringx = "0".ToCharArray();
			success = true;
		}else{
			characterReference = new CharacterReference();

			if(IsInteger(basex)){
				success = true;

				stringx = new char [0];

				maximumDigits = nGetMaximumDigitsForBase(basex);

				digitPosition = nGetFirstDigitPosition(decimalx, basex);

				decimalx = (double)Round(decimalx*Pow(basex, maximumDigits - digitPosition - 1d));

				hasPrintedPoint = false;

				if(!isPositive){
					stringx = AppendCharacter(stringx, '-');
				}

				/* Print leading zeros.*/
				if(digitPosition < 0d){
					stringx = AppendCharacter(stringx, '0');
					stringx = AppendCharacter(stringx, '.');
					hasPrintedPoint = true;
					for(i = 0d; i < -digitPosition - 1d; i = i + 1d){
						stringx = AppendCharacter(stringx, '0');
					}
				}

				/* Print number.*/
				for(i = 0d; i < maximumDigits && success; i = i + 1d){
					d = Floor(decimalx/Pow(basex, maximumDigits - i - 1d));

					if(d >= basex){
						d = basex - 1d;
					}

					if(!hasPrintedPoint && digitPosition - i + 1d == 0d){
						if(decimalx != 0d){
							stringx = AppendCharacter(stringx, '.');
						}
						hasPrintedPoint = true;
					}

					if(decimalx == 0d && hasPrintedPoint){
					}else{
						success = nGetSingleDigitCharacterFromNumberWithCheck(d, basex, characterReference);
						if(success){
							c = characterReference.characterValue;
							stringx = AppendCharacter(stringx, c);
						}
					}

					if(success){
						decimalx = decimalx - d*Pow(basex, maximumDigits - i - 1d);
					}
				}

				if(success){
					/* Print trailing zeros.*/
					for(i = 0d; i < digitPosition - maximumDigits + 1d; i = i + 1d){
						stringx = AppendCharacter(stringx, '0');
					}

					stringReference.stringx = stringx;
				}
			}else{
				success = false;
			}
		}

		/* Done*/
		return success;
	}


	public static double nGetMaximumDigitsForBase(double basex){
		double t;

		t = Pow(10d, 15d);
		return Floor(Log10(t)/Log10(basex));
	}


	public static double nGetFirstDigitPosition(double decimalx, double basex){
		double power;
		double t;

		power = Ceiling(Log10(decimalx)/Log10(basex));

		t = decimalx*Pow(basex, -power);
		if(t < basex && t >= 1d){
		}else if(t >= basex){
			power = power + 1d;
		}else if(t < 1d){
			power = power - 1d;
		}

		return power;
	}


	public static bool nGetSingleDigitCharacterFromNumberWithCheck(double c, double basex, CharacterReference characterReference){
		char [] numberTable;
		bool success;

		numberTable = nGetDigitCharacterTable();

		if(c < basex || c < numberTable.Length){
			success = true;
			characterReference.characterValue = numberTable[(int)(c)];
		}else{
			success = false;
		}

		return success;
	}


	public static char [] nGetDigitCharacterTable(){
		char [] numberTable;

		numberTable = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ".ToCharArray();

		return numberTable;
	}


	public static bool nCreateNumberFromDecimalStringWithCheck(char [] stringx, NumberReference decimalReference, StringReference errorMessage){
		return nCreateNumberFromStringWithCheck(stringx, 10d, decimalReference, errorMessage);
	}


	public static double nCreateNumberFromDecimalString(char [] stringx){
		NumberReference doubleReference;
		StringReference stringReference;
		double number;

		doubleReference = CreateNumberReference(0d);
		stringReference = CreateStringReference("".ToCharArray());
		nCreateNumberFromStringWithCheck(stringx, 10d, doubleReference, stringReference);
		number = doubleReference.numberValue;

		delete(doubleReference);
		delete(stringReference);

		return number;
	}


	public static bool nCreateNumberFromStringWithCheck(char [] stringx, double basex, NumberReference numberReference, StringReference errorMessage){
		bool success;
		BooleanReference numberIsPositive, exponentIsPositive;
		NumberArrayReference beforePoint, afterPoint, exponent;

		numberIsPositive = CreateBooleanReference(true);
		exponentIsPositive = CreateBooleanReference(true);
		beforePoint = new NumberArrayReference();
		afterPoint = new NumberArrayReference();
		exponent = new NumberArrayReference();

		if(basex >= 2d && basex <= 36d){
			success = nExtractPartsFromNumberString(stringx, basex, numberIsPositive, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessage);

			if(success){
				numberReference.numberValue = nCreateNumberFromParts(basex, numberIsPositive.booleanValue, beforePoint.numberArray, afterPoint.numberArray, exponentIsPositive.booleanValue, exponent.numberArray);
			}
		}else{
			success = false;
			errorMessage.stringx = "Base must be from 2 to 36.".ToCharArray();
		}

		return success;
	}


	public static double nCreateNumberFromParts(double basex, bool numberIsPositive, double [] beforePoint, double [] afterPoint, bool exponentIsPositive, double [] exponent){
		double n, i, p, e;

		n = 0d;

		for(i = 0d; i < beforePoint.Length; i = i + 1d){
			p = beforePoint[(int)(beforePoint.Length - i - 1d)];

			n = n + p*Pow(basex, i);
		}

		for(i = 0d; i < afterPoint.Length; i = i + 1d){
			p = afterPoint[(int)(i)];

			n = n + p*Pow(basex, -(i + 1d));
		}

		if(exponent.Length > 0d){
			e = 0d;
			for(i = 0d; i < exponent.Length; i = i + 1d){
				p = exponent[(int)(exponent.Length - i - 1d)];

				e = e + p*Pow(basex, i);
			}

			if(!exponentIsPositive){
				e = -e;
			}

			n = n*Pow(basex, e);
		}

		if(!numberIsPositive){
			n = -n;
		}

		return n;
	}


	public static bool nExtractPartsFromNumberString(char [] n, double basex, BooleanReference numberIsPositive, NumberArrayReference beforePoint, NumberArrayReference afterPoint, BooleanReference exponentIsPositive, NumberArrayReference exponent, StringReference errorMessages){
		double i;
		bool success;

		i = 0d;

		if(i < n.Length){
			if(n[(int)(i)] == '-'){
				numberIsPositive.booleanValue = false;
				i = i + 1d;
			}else if(n[(int)(i)] == '+'){
				numberIsPositive.booleanValue = true;
				i = i + 1d;
			}

			success = nExtractPartsFromNumberStringFromSign(n, basex, i, beforePoint, afterPoint, exponentIsPositive, exponent, errorMessages);
		}else{
			success = false;
			errorMessages.stringx = "Number cannot have length zero.".ToCharArray();
		}

		return success;
	}


	public static bool nExtractPartsFromNumberStringFromSign(char [] n, double basex, double i, NumberArrayReference beforePoint, NumberArrayReference afterPoint, BooleanReference exponentIsPositive, NumberArrayReference exponent, StringReference errorMessages){
		bool success, done;
		double count, j;

		done = false;
		count = 0d;
		for(; i + count < n.Length && !done; ){
			if(nCharacterIsNumberCharacterInBase(n[(int)(i + count)], basex)){
				count = count + 1d;
			}else{
				done = true;
			}
		}

		if(count >= 1d){
			beforePoint.numberArray = new double [(int)(count)];

			for(j = 0d; j < count; j = j + 1d){
				beforePoint.numberArray[(int)(j)] = nGetNumberFromNumberCharacterForBase(n[(int)(i + j)], basex);
			}

			i = i + count;

			if(i < n.Length){
				success = nExtractPartsFromNumberStringFromPointOrExponent(n, basex, i, afterPoint, exponentIsPositive, exponent, errorMessages);
			}else{
				afterPoint.numberArray = new double [0];
				exponent.numberArray = new double [0];
				success = true;
			}
		}else{
			success = false;
			errorMessages.stringx = "Number must have at least one number after the optional sign.".ToCharArray();
		}

		return success;
	}


	public static bool nExtractPartsFromNumberStringFromPointOrExponent(char [] n, double basex, double i, NumberArrayReference afterPoint, BooleanReference exponentIsPositive, NumberArrayReference exponent, StringReference errorMessages){
		bool success, done;
		double count, j;

		if(n[(int)(i)] == '.'){
			i = i + 1d;

			if(i < n.Length){
				done = false;
				count = 0d;
				for(; i + count < n.Length && !done; ){
					if(nCharacterIsNumberCharacterInBase(n[(int)(i + count)], basex)){
						count = count + 1d;
					}else{
						done = true;
					}
				}

				if(count >= 1d){
					afterPoint.numberArray = new double [(int)(count)];

					for(j = 0d; j < count; j = j + 1d){
						afterPoint.numberArray[(int)(j)] = nGetNumberFromNumberCharacterForBase(n[(int)(i + j)], basex);
					}

					i = i + count;

					if(i < n.Length){
						success = nExtractPartsFromNumberStringFromExponent(n, basex, i, exponentIsPositive, exponent, errorMessages);
					}else{
						exponent.numberArray = new double [0];
						success = true;
					}
				}else{
					success = false;
					errorMessages.stringx = "There must be at least one digit after the decimal point.".ToCharArray();
				}
			}else{
				success = false;
				errorMessages.stringx = "There must be at least one digit after the decimal point.".ToCharArray();
			}
		}else if(basex <= 14d && (n[(int)(i)] == 'e' || n[(int)(i)] == 'E')){
			if(i < n.Length){
				success = nExtractPartsFromNumberStringFromExponent(n, basex, i, exponentIsPositive, exponent, errorMessages);
				afterPoint.numberArray = new double [0];
			}else{
				success = false;
				errorMessages.stringx = "There must be at least one digit after the exponent.".ToCharArray();
			}
		}else{
			success = false;
			errorMessages.stringx = "Expected decimal point or exponent symbol.".ToCharArray();
		}

		return success;
	}


	public static bool nExtractPartsFromNumberStringFromExponent(char [] n, double basex, double i, BooleanReference exponentIsPositive, NumberArrayReference exponent, StringReference errorMessages){
		bool success, done;
		double count, j;

		if(basex <= 14d && (n[(int)(i)] == 'e' || n[(int)(i)] == 'E')){
			i = i + 1d;

			if(i < n.Length){
				if(n[(int)(i)] == '-'){
					exponentIsPositive.booleanValue = false;
					i = i + 1d;
				}else if(n[(int)(i)] == '+'){
					exponentIsPositive.booleanValue = true;
					i = i + 1d;
				}

				if(i < n.Length){
					done = false;
					count = 0d;
					for(; i + count < n.Length && !done; ){
						if(nCharacterIsNumberCharacterInBase(n[(int)(i + count)], basex)){
							count = count + 1d;
						}else{
							done = true;
						}
					}

					if(count >= 1d){
						exponent.numberArray = new double [(int)(count)];

						for(j = 0d; j < count; j = j + 1d){
							exponent.numberArray[(int)(j)] = nGetNumberFromNumberCharacterForBase(n[(int)(i + j)], basex);
						}

						i = i + count;

						if(i == n.Length){
							success = true;
						}else{
							success = false;
							errorMessages.stringx = "There cannot be any characters past the exponent of the number.".ToCharArray();
						}
					}else{
						success = false;
						errorMessages.stringx = "There must be at least one digit after the decimal point.".ToCharArray();
					}
				}else{
					success = false;
					errorMessages.stringx = "There must be at least one digit after the exponent symbol.".ToCharArray();
				}
			}else{
				success = false;
				errorMessages.stringx = "There must be at least one digit after the exponent symbol.".ToCharArray();
			}
		}else{
			success = false;
			errorMessages.stringx = "Expected exponent symbol.".ToCharArray();
		}

		return success;
	}


	public static double nGetNumberFromNumberCharacterForBase(char c, double basex){
		char [] numberTable;
		double i;
		double position;

		numberTable = nGetDigitCharacterTable();
		position = 0d;

		for(i = 0d; i < basex; i = i + 1d){
			if(numberTable[(int)(i)] == c){
				position = i;
			}
		}

		return position;
	}


	public static bool nCharacterIsNumberCharacterInBase(char c, double basex){
		char [] numberTable;
		double i;
		bool found;

		numberTable = nGetDigitCharacterTable();
		found = false;

		for(i = 0d; i < basex; i = i + 1d){
			if(numberTable[(int)(i)] == c){
				found = true;
			}
		}

		return found;
	}


	public static double [] nStringToNumberArray(char [] str){
		NumberArrayReference numberArrayReference;
		StringReference stringReference;
		double [] numbers;

		numberArrayReference = new NumberArrayReference();
		stringReference = new StringReference();

		nStringToNumberArrayWithCheck(str, numberArrayReference, stringReference);

		numbers = numberArrayReference.numberArray;

		delete(numberArrayReference);
		delete(stringReference);

		return numbers;
	}


	public static bool nStringToNumberArrayWithCheck(char [] str, NumberArrayReference numberArrayReference, StringReference errorMessage){
		StringReference [] numberStrings;
		double [] numbers;
		double i;
		char [] numberString, trimmedNumberString;
		bool success;
		NumberReference numberReference;

		numberStrings = SplitByString(str, ",".ToCharArray());

		numbers = new double [(int)(numberStrings.Length)];
		success = true;
		numberReference = new NumberReference();

		for(i = 0d; i < numberStrings.Length; i = i + 1d){
			numberString = numberStrings[(int)(i)].stringx;
			trimmedNumberString = Trim(numberString);
			success = nCreateNumberFromDecimalStringWithCheck(trimmedNumberString, numberReference, errorMessage);
			numbers[(int)(i)] = numberReference.numberValue;

			FreeStringReference(numberStrings[(int)(i)]);
			delete(trimmedNumberString);
		}

		delete(numberStrings);
		delete(numberReference);

		numberArrayReference.numberArray = numbers;

		return success;
	}


	public static double [] AddNumber(double [] list, double a){
		double [] newlist;
		double i;

		newlist = new double [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddNumberRef(NumberArrayReference list, double i){
		list.numberArray = AddNumber(list.numberArray, i);
	}


	public static double [] RemoveNumber(double [] list, double n){
		double [] newlist;
		double i;

		newlist = new double [(int)(list.Length - 1d)];

		if(n >= 0d && n < list.Length){
			for(i = 0d; i < list.Length; i = i + 1d){
				if(i < n){
					newlist[(int)(i)] = list[(int)(i)];
				}
				if(i > n){
					newlist[(int)(i - 1d)] = list[(int)(i)];
				}
			}

			delete(list);
		}else{
			delete(newlist);
		}
		
		return newlist;
	}


	public static double GetNumberRef(NumberArrayReference list, double i){
		return list.numberArray[(int)(i)];
	}


	public static void RemoveNumberRef(NumberArrayReference list, double i){
		list.numberArray = RemoveNumber(list.numberArray, i);
	}


	public static StringReference [] AddString(StringReference [] list, StringReference a){
		StringReference [] newlist;
		double i;

		newlist = new StringReference [(int)(list.Length + 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddStringRef(StringArrayReference list, StringReference i){
		list.stringArray = AddString(list.stringArray, i);
	}


	public static StringReference [] RemoveString(StringReference [] list, double n){
		StringReference [] newlist;
		double i;

		newlist = new StringReference [(int)(list.Length - 1d)];

		if(n >= 0d && n < list.Length){
			for(i = 0d; i < list.Length; i = i + 1d){
				if(i < n){
					newlist[(int)(i)] = list[(int)(i)];
				}
				if(i > n){
					newlist[(int)(i - 1d)] = list[(int)(i)];
				}
			}

			delete(list);
		}else{
			delete(newlist);
		}
		
		return newlist;
	}


	public static StringReference GetStringRef(StringArrayReference list, double i){
		return list.stringArray[(int)(i)];
	}


	public static void RemoveStringRef(StringArrayReference list, double i){
		list.stringArray = RemoveString(list.stringArray, i);
	}


	public static bool [] AddBoolean(bool [] list, bool a){
		bool [] newlist;
		double i;

		newlist = new bool [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddBooleanRef(BooleanArrayReference list, bool i){
		list.booleanArray = AddBoolean(list.booleanArray, i);
	}


	public static bool [] RemoveBoolean(bool [] list, double n){
		bool [] newlist;
		double i;

		newlist = new bool [(int)(list.Length - 1d)];

		if(n >= 0d && n < list.Length){
			for(i = 0d; i < list.Length; i = i + 1d){
				if(i < n){
					newlist[(int)(i)] = list[(int)(i)];
				}
				if(i > n){
					newlist[(int)(i - 1d)] = list[(int)(i)];
				}
			}

			delete(list);
		}else{
			delete(newlist);
		}
		
		return newlist;
	}


	public static bool GetBooleanRef(BooleanArrayReference list, double i){
		return list.booleanArray[(int)(i)];
	}


	public static void RemoveDecimalRef(BooleanArrayReference list, double i){
		list.booleanArray = RemoveBoolean(list.booleanArray, i);
	}


	public static LinkedListStrings CreateLinkedListString(){
		LinkedListStrings ll;

		ll = new LinkedListStrings();
		ll.first = new LinkedListNodeStrings();
		ll.last = ll.first;
		ll.last.end = true;

		return ll;
	}


	public static void LinkedListAddString(LinkedListStrings ll, char [] value){
		ll.last.end = false;
		ll.last.value = value;
		ll.last.next = new LinkedListNodeStrings();
		ll.last.next.end = true;
		ll.last = ll.last.next;
	}


	public static StringReference [] LinkedListStringsToArray(LinkedListStrings ll){
		StringReference [] array;
		double length, i;
		LinkedListNodeStrings node;

		node = ll.first;

		length = LinkedListStringsLength(ll);

		array = new StringReference [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			array[(int)(i)] = new StringReference();
			array[(int)(i)].stringx = node.value;
			node = node.next;
		}

		return array;
	}


	public static double LinkedListStringsLength(LinkedListStrings ll){
		double l;
		LinkedListNodeStrings node;

		l = 0d;
		node = ll.first;
		for(; !node.end; ){
			node = node.next;
			l = l + 1d;
		}

		return l;
	}


	public static void FreeLinkedListString(LinkedListStrings ll){
		LinkedListNodeStrings node, prev;

		node = ll.first;

		for(; !node.end; ){
			prev = node;
			node = node.next;
			delete(prev);
		}

		delete(node);
	}


	public static LinkedListNumbers CreateLinkedListNumbers(){
		LinkedListNumbers ll;

		ll = new LinkedListNumbers();
		ll.first = new LinkedListNodeNumbers();
		ll.last = ll.first;
		ll.last.end = true;

		return ll;
	}


	public static LinkedListNumbers [] CreateLinkedListNumbersArray(double length){
		LinkedListNumbers [] lls;
		double i;

		lls = new LinkedListNumbers [(int)(length)];
		for(i = 0d; i < lls.Length; i = i + 1d){
			lls[(int)(i)] = CreateLinkedListNumbers();
		}

		return lls;
	}


	public static void LinkedListAddNumber(LinkedListNumbers ll, double value){
		ll.last.end = false;
		ll.last.value = value;
		ll.last.next = new LinkedListNodeNumbers();
		ll.last.next.end = true;
		ll.last = ll.last.next;
	}


	public static double LinkedListNumbersLength(LinkedListNumbers ll){
		double l;
		LinkedListNodeNumbers node;

		l = 0d;
		node = ll.first;
		for(; !node.end; ){
			node = node.next;
			l = l + 1d;
		}

		return l;
	}


	public static double LinkedListNumbersIndex(LinkedListNumbers ll, double index){
		double i;
		LinkedListNodeNumbers node;

		node = ll.first;
		for(i = 0d; i < index; i = i + 1d){
			node = node.next;
		}

		return node.value;
	}


	public static void LinkedListInsertNumber(LinkedListNumbers ll, double index, double value){
		double i;
		LinkedListNodeNumbers node, tmp;

		if(index == 0d){
			tmp = ll.first;
			ll.first = new LinkedListNodeNumbers();
			ll.first.next = tmp;
			ll.first.value = value;
			ll.first.end = false;
		}else{
			node = ll.first;
			for(i = 0d; i < index - 1d; i = i + 1d){
				node = node.next;
			}

			tmp = node.next;
			node.next = new LinkedListNodeNumbers();
			node.next.next = tmp;
			node.next.value = value;
			node.next.end = false;
		}
	}


	public static void LinkedListSet(LinkedListNumbers ll, double index, double value){
		double i;
		LinkedListNodeNumbers node;

		node = ll.first;
		for(i = 0d; i < index; i = i + 1d){
			node = node.next;
		}

		node.next.value = value;
	}


	public static void LinkedListRemoveNumber(LinkedListNumbers ll, double index){
		double i;
		LinkedListNodeNumbers node, prev;

		node = ll.first;
		prev = ll.first;

		for(i = 0d; i < index; i = i + 1d){
			prev = node;
			node = node.next;
		}

		if(index == 0d){
			ll.first = prev.next;
		}
		if(!prev.next.end){
			prev.next = prev.next.next;
		}
	}


	public static void FreeLinkedListNumbers(LinkedListNumbers ll){
		LinkedListNodeNumbers node, prev;

		node = ll.first;

		for(; !node.end; ){
			prev = node;
			node = node.next;
			delete(prev);
		}

		delete(node);
	}


	public static void FreeLinkedListNumbersArray(LinkedListNumbers [] lls){
		double i;

		for(i = 0d; i < lls.Length; i = i + 1d){
			FreeLinkedListNumbers(lls[(int)(i)]);
		}
		delete(lls);
	}


	public static double [] LinkedListNumbersToArray(LinkedListNumbers ll){
		double [] array;
		double length, i;
		LinkedListNodeNumbers node;

		node = ll.first;

		length = LinkedListNumbersLength(ll);

		array = new double [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			array[(int)(i)] = node.value;
			node = node.next;
		}

		return array;
	}


	public static LinkedListNumbers ArrayToLinkedListNumbers(double [] array){
		LinkedListNumbers ll;
		double i;

		ll = CreateLinkedListNumbers();

		for(i = 0d; i < array.Length; i = i + 1d){
			LinkedListAddNumber(ll, array[(int)(i)]);
		}

		return ll;
	}


	public static bool LinkedListNumbersEqual(LinkedListNumbers a, LinkedListNumbers b){
		bool equal, done;
		LinkedListNodeNumbers an, bn;

		an = a.first;
		bn = b.first;

		equal = true;
		done = false;
		for(; equal && !done; ){
			if(an.end == bn.end){
				if(an.end){
					done = true;
				}else if(an.value == bn.value){
					an = an.next;
					bn = bn.next;
				}else{
					equal = false;
				}
			}else{
				equal = false;
			}
		}

		return equal;
	}


	public static LinkedListCharacters CreateLinkedListCharacter(){
		LinkedListCharacters ll;

		ll = new LinkedListCharacters();
		ll.first = new LinkedListNodeCharacters();
		ll.last = ll.first;
		ll.last.end = true;

		return ll;
	}


	public static void LinkedListAddCharacter(LinkedListCharacters ll, char value){
		ll.last.end = false;
		ll.last.value = value;
		ll.last.next = new LinkedListNodeCharacters();
		ll.last.next.end = true;
		ll.last = ll.last.next;
	}


	public static char [] LinkedListCharactersToArray(LinkedListCharacters ll){
		char [] array;
		double length, i;
		LinkedListNodeCharacters node;

		node = ll.first;

		length = LinkedListCharactersLength(ll);

		array = new char [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			array[(int)(i)] = node.value;
			node = node.next;
		}

		return array;
	}


	public static double LinkedListCharactersLength(LinkedListCharacters ll){
		double l;
		LinkedListNodeCharacters node;

		l = 0d;
		node = ll.first;
		for(; !node.end; ){
			node = node.next;
			l = l + 1d;
		}

		return l;
	}


	public static void FreeLinkedListCharacter(LinkedListCharacters ll){
		LinkedListNodeCharacters node, prev;

		node = ll.first;

		for(; !node.end; ){
			prev = node;
			node = node.next;
			delete(prev);
		}

		delete(node);
	}


	public static DynamicArrayNumbers CreateDynamicArrayNumbers(){
		DynamicArrayNumbers da;

		da = new DynamicArrayNumbers();
		da.array = new double [10];
		da.length = 0d;

		return da;
	}


	public static DynamicArrayNumbers CreateDynamicArrayNumbersWithInitialCapacity(double capacity){
		DynamicArrayNumbers da;

		da = new DynamicArrayNumbers();
		da.array = new double [(int)(capacity)];
		da.length = 0d;

		return da;
	}


	public static void DynamicArrayAddNumber(DynamicArrayNumbers da, double value){
		if(da.length == da.array.Length){
			DynamicArrayNumbersIncreaseSize(da);
		}

		da.array[(int)(da.length)] = value;
		da.length = da.length + 1d;
	}


	public static void DynamicArrayNumbersIncreaseSize(DynamicArrayNumbers da){
		double newLength, i;
		double [] newArray;

		newLength = (double)Round(da.array.Length*3d/2d);
		newArray = new double [(int)(newLength)];

		for(i = 0d; i < da.array.Length; i = i + 1d){
			newArray[(int)(i)] = da.array[(int)(i)];
		}

		delete(da.array);

		da.array = newArray;
	}


	public static bool DynamicArrayNumbersDecreaseSizeNecessary(DynamicArrayNumbers da){
		bool needsDecrease;

		needsDecrease = false;

		if(da.length > 10d){
			needsDecrease = da.length <= (double)Round(da.array.Length*2d/3d);
		}

		return needsDecrease;
	}


	public static void DynamicArrayNumbersDecreaseSize(DynamicArrayNumbers da){
		double newLength, i;
		double [] newArray;

		newLength = (double)Round(da.array.Length*2d/3d);
		newArray = new double [(int)(newLength)];

		for(i = 0d; i < da.array.Length; i = i + 1d){
			newArray[(int)(i)] = da.array[(int)(i)];
		}

		delete(da.array);

		da.array = newArray;
	}


	public static double DynamicArrayNumbersIndex(DynamicArrayNumbers da, double index){
		return da.array[(int)(index)];
	}


	public static double DynamicArrayNumbersLength(DynamicArrayNumbers da){
		return da.length;
	}


	public static void DynamicArrayInsertNumber(DynamicArrayNumbers da, double index, double value){
		double i;

		if(da.length == da.array.Length){
			DynamicArrayNumbersIncreaseSize(da);
		}

		for(i = da.length; i > index; i = i - 1d){
			da.array[(int)(i)] = da.array[(int)(i - 1d)];
		}

		da.array[(int)(index)] = value;

		da.length = da.length + 1d;
	}


	public static void DynamicArraySet(DynamicArrayNumbers da, double index, double value){
		da.array[(int)(index)] = value;
	}


	public static void DynamicArrayRemoveNumber(DynamicArrayNumbers da, double index){
		double i;

		for(i = index; i < da.length - 1d; i = i + 1d){
			da.array[(int)(i)] = da.array[(int)(i + 1d)];
		}

		da.length = da.length - 1d;

		if(DynamicArrayNumbersDecreaseSizeNecessary(da)){
			DynamicArrayNumbersDecreaseSize(da);
		}
	}


	public static void FreeDynamicArrayNumbers(DynamicArrayNumbers da){
		delete(da.array);
		delete(da);
	}


	public static double [] DynamicArrayNumbersToArray(DynamicArrayNumbers da){
		double [] array;
		double i;

		array = new double [(int)(da.length)];

		for(i = 0d; i < da.length; i = i + 1d){
			array[(int)(i)] = da.array[(int)(i)];
		}

		return array;
	}


	public static DynamicArrayNumbers ArrayToDynamicArrayNumbersWithOptimalSize(double [] array){
		DynamicArrayNumbers da;
		double i;
		double c, n, newCapacity;

		/*
         c = 10*(3/2)^n
         log(c) = log(10*(3/2)^n)
         log(c) = log(10) + log((3/2)^n)
         log(c) = 1 + log((3/2)^n)
         log(c) - 1 = log((3/2)^n)
         log(c) - 1 = n*log(3/2)
         n = (log(c) - 1)/log(3/2)
        */
		c = array.Length;
		n = (Log(c) - 1d)/Log(3d/2d);
		newCapacity = Floor(n) + 1d;

		da = CreateDynamicArrayNumbersWithInitialCapacity(newCapacity);

		for(i = 0d; i < array.Length; i = i + 1d){
			da.array[(int)(i)] = array[(int)(i)];
		}

		return da;
	}


	public static DynamicArrayNumbers ArrayToDynamicArrayNumbers(double [] array){
		DynamicArrayNumbers da;

		da = new DynamicArrayNumbers();
		da.array = CopyNumberArray(array);
		da.length = array.Length;

		return da;
	}


	public static bool DynamicArrayNumbersEqual(DynamicArrayNumbers a, DynamicArrayNumbers b){
		bool equal;
		double i;

		equal = true;
		if(a.length == b.length){
			for(i = 0d; i < a.length && equal; i = i + 1d){
				if(a.array[(int)(i)] != b.array[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static LinkedListNumbers DynamicArrayNumbersToLinkedList(DynamicArrayNumbers da){
		LinkedListNumbers ll;
		double i;

		ll = CreateLinkedListNumbers();

		for(i = 0d; i < da.length; i = i + 1d){
			LinkedListAddNumber(ll, da.array[(int)(i)]);
		}

		return ll;
	}


	public static DynamicArrayNumbers LinkedListToDynamicArrayNumbers(LinkedListNumbers ll){
		DynamicArrayNumbers da;
		double i;
		LinkedListNodeNumbers node;

		node = ll.first;

		da = new DynamicArrayNumbers();
		da.length = LinkedListNumbersLength(ll);

		da.array = new double [(int)(da.length)];

		for(i = 0d; i < da.length; i = i + 1d){
			da.array[(int)(i)] = node.value;
			node = node.next;
		}

		return da;
	}


	public static char [] AddCharacter(char [] list, char a){
		char [] newlist;
		double i;

		newlist = new char [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddCharacterRef(StringReference list, char i){
		list.stringx = AddCharacter(list.stringx, i);
	}


	public static char [] RemoveCharacter(char [] list, double n){
		char [] newlist;
		double i;

		newlist = new char [(int)(list.Length - 1d)];

		if(n >= 0d && n < list.Length){
			for(i = 0d; i < list.Length; i = i + 1d){
				if(i < n){
					newlist[(int)(i)] = list[(int)(i)];
				}
				if(i > n){
					newlist[(int)(i - 1d)] = list[(int)(i)];
				}
			}

			delete(list);
		}else{
			delete(newlist);
		}

		return newlist;
	}


	public static char GetCharacterRef(StringReference list, double i){
		return list.stringx[(int)(i)];
	}


	public static void RemoveCharacterRef(StringReference list, double i){
		list.stringx = RemoveCharacter(list.stringx, i);
	}


	public static double [] StringToNumberArray(char [] stringx){
		double i;
		double [] array;

		array = new double [(int)(stringx.Length)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			array[(int)(i)] = stringx[(int)(i)];
		}
		return array;
	}


	public static char [] NumberArrayToString(double [] array){
		double i;
		char [] stringx;

		stringx = new char [(int)(array.Length)];

		for(i = 0d; i < array.Length; i = i + 1d){
			stringx[(int)(i)] = (char)(array[(int)(i)]);
		}
		return stringx;
	}


	public static bool NumberArraysEqual(double [] a, double [] b){
		bool equal;
		double i;

		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static bool BooleanArraysEqual(bool [] a, bool [] b){
		bool equal;
		double i;

		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static bool StringsEqual(char [] a, char [] b){
		bool equal;
		double i;

		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static void FillNumberArray(double [] a, double value){
		double i;

		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static void FillString(char [] a, char value){
		double i;

		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static void FillBooleanArray(bool [] a, bool value){
		double i;

		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static bool FillNumberArrayRange(double [] a, double value, double from, double to){
		double i, length;
		bool success;

		if(from >= 0d && from <= a.Length && to >= 0d && to <= a.Length && from <= to){
			length = to - from;
			for(i = 0d; i < length; i = i + 1d){
				a[(int)(from + i)] = value;
			}

			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool FillBooleanArrayRange(bool [] a, bool value, double from, double to){
		double i, length;
		bool success;

		if(from >= 0d && from <= a.Length && to >= 0d && to <= a.Length && from <= to){
			length = to - from;
			for(i = 0d; i < length; i = i + 1d){
				a[(int)(from + i)] = value;
			}

			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool FillStringRange(char [] a, char value, double from, double to){
		double i, length;
		bool success;

		if(from >= 0d && from <= a.Length && to >= 0d && to <= a.Length && from <= to){
			length = to - from;
			for(i = 0d; i < length; i = i + 1d){
				a[(int)(from + i)] = value;
			}

			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static double [] CopyNumberArray(double [] a){
		double i;
		double [] n;

		n = new double [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static bool [] CopyBooleanArray(bool [] a){
		double i;
		bool [] n;

		n = new bool [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static char [] CopyString(char [] a){
		double i;
		char [] n;

		n = new char [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static bool CopyNumberArrayRange(double [] a, double from, double to, NumberArrayReference copyReference){
		double i, length;
		double [] n;
		bool success;

		if(from >= 0d && from <= a.Length && to >= 0d && to <= a.Length && from <= to){
			length = to - from;
			n = new double [(int)(length)];

			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}

			copyReference.numberArray = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool CopyBooleanArrayRange(bool [] a, double from, double to, BooleanArrayReference copyReference){
		double i, length;
		bool [] n;
		bool success;

		if(from >= 0d && from <= a.Length && to >= 0d && to <= a.Length && from <= to){
			length = to - from;
			n = new bool [(int)(length)];

			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}

			copyReference.booleanArray = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool CopyStringRange(char [] a, double from, double to, StringReference copyReference){
		double i, length;
		char [] n;
		bool success;

		if(from >= 0d && from <= a.Length && to >= 0d && to <= a.Length && from <= to){
			length = to - from;
			n = new char [(int)(length)];

			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}

			copyReference.stringx = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool IsLastElement(double length, double index){
		return index + 1d == length;
	}


	public static double [] CreateNumberArray(double length, double value){
		double [] array;

		array = new double [(int)(length)];
		FillNumberArray(array, value);

		return array;
	}


	public static bool [] CreateBooleanArray(double length, bool value){
		bool [] array;

		array = new bool [(int)(length)];
		FillBooleanArray(array, value);

		return array;
	}


	public static char [] CreateString(double length, char value){
		char [] array;

		array = new char [(int)(length)];
		FillString(array, value);

		return array;
	}


	public static void SwapElementsOfNumberArray(double [] A, double ai, double bi){
		double tmp;

		tmp = A[(int)(ai)];
		A[(int)(ai)] = A[(int)(bi)];
		A[(int)(bi)] = tmp;
	}


	public static void SwapElementsOfStringArray(StringArrayReference A, double ai, double bi){
		StringReference tmp;

		tmp = A.stringArray[(int)(ai)];
		A.stringArray[(int)(ai)] = A.stringArray[(int)(bi)];
		A.stringArray[(int)(bi)] = tmp;
	}


	public static void ReverseNumberArray(double [] array){
		double i;

		for(i = 0d; i < array.Length/2d; i = i + 1d){
			SwapElementsOfNumberArray(array, i, array.Length - i - 1d);
		}
	}


	public static void WriteStringToStingStream(char [] stream, NumberReference index, char [] src){
		double i;

		for(i = 0d; i < src.Length; i = i + 1d){
			stream[(int)(index.numberValue + i)] = src[(int)(i)];
		}
		index.numberValue = index.numberValue + src.Length;
	}


	public static void WriteCharacterToStingStream(char [] stream, NumberReference index, char src){
		stream[(int)(index.numberValue)] = src;
		index.numberValue = index.numberValue + 1d;
	}


	public static void WriteBooleanToStingStream(char [] stream, NumberReference index, bool src){
		if(src){
			WriteStringToStingStream(stream, index, "true".ToCharArray());
		}else{
			WriteStringToStingStream(stream, index, "false".ToCharArray());
		}
	}


	public static bool SubstringWithCheck(char [] stringx, double from, double to, StringReference stringReference){
		bool success;

		if(from >= 0d && from <= stringx.Length && to >= 0d && to <= stringx.Length && from <= to){
			stringReference.stringx = Substring(stringx, from, to);
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static char [] Substring(char [] stringx, double from, double to){
		char [] n;
		double i, length;

		length = to - from;

		n = new char [(int)(length)];

		for(i = from; i < to; i = i + 1d){
			n[(int)(i - from)] = stringx[(int)(i)];
		}

		return n;
	}


	public static char [] AppendString(char [] s1, char [] s2){
		char [] newString;

		newString = ConcatenateString(s1, s2);

		delete(s1);

		return newString;
	}


	public static char [] ConcatenateString(char [] s1, char [] s2){
		char [] newString;
		double i;

		newString = new char [(int)(s1.Length + s2.Length)];

		for(i = 0d; i < s1.Length; i = i + 1d){
			newString[(int)(i)] = s1[(int)(i)];
		}

		for(i = 0d; i < s2.Length; i = i + 1d){
			newString[(int)(s1.Length + i)] = s2[(int)(i)];
		}

		return newString;
	}


	public static char [] AppendCharacter(char [] stringx, char c){
		char [] newString;

		newString = ConcatenateCharacter(stringx, c);

		delete(stringx);

		return newString;
	}


	public static char [] ConcatenateCharacter(char [] stringx, char c){
		char [] newString;
		double i;
		newString = new char [(int)(stringx.Length + 1d)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			newString[(int)(i)] = stringx[(int)(i)];
		}

		newString[(int)(stringx.Length)] = c;

		return newString;
	}


	public static StringReference [] SplitByCharacter(char [] toSplit, char splitBy){
		StringReference [] split;
		char [] stringToSplitBy;

		stringToSplitBy = new char [1];
		stringToSplitBy[0] = splitBy;

		split = SplitByString(toSplit, stringToSplitBy);

		delete(stringToSplitBy);

		return split;
	}


	public static bool IndexOfCharacter(char [] stringx, char character, NumberReference indexReference){
		double i;
		bool found;

		found = false;
		for(i = 0d; i < stringx.Length && !found; i = i + 1d){
			if(stringx[(int)(i)] == character){
				found = true;
				indexReference.numberValue = i;
			}
		}

		return found;
	}


	public static bool SubstringEqualsWithCheck(char [] stringx, double from, char [] substring, BooleanReference equalsReference){
		bool success;

		if(from < stringx.Length){
			success = true;
			equalsReference.booleanValue = SubstringEquals(stringx, from, substring);
		}else{
			success = false;
		}

		return success;
	}


	public static bool SubstringEquals(char [] stringx, double from, char [] substring){
		double i;
		bool equal;

		equal = true;
		for(i = 0d; i < substring.Length && equal; i = i + 1d){
			if(stringx[(int)(from + i)] != substring[(int)(i)]){
				equal = false;
			}
		}

		return equal;
	}


	public static bool IndexOfString(char [] stringx, char [] substring, NumberReference indexReference){
		double i;
		bool found;

		found = false;
		for(i = 0d; i < stringx.Length - substring.Length + 1d && !found; i = i + 1d){
			if(SubstringEquals(stringx, i, substring)){
				found = true;
				indexReference.numberValue = i;
			}
		}

		return found;
	}


	public static bool ContainsCharacter(char [] stringx, char character){
		double i;
		bool found;

		found = false;
		for(i = 0d; i < stringx.Length && !found; i = i + 1d){
			if(stringx[(int)(i)] == character){
				found = true;
			}
		}

		return found;
	}


	public static bool ContainsString(char [] stringx, char [] substring){
		return IndexOfString(stringx, substring, new NumberReference());
	}


	public static void ToUpperCase(char [] stringx){
		double i;

		for(i = 0d; i < stringx.Length; i = i + 1d){
			stringx[(int)(i)] = charToUpperCase(stringx[(int)(i)]);
		}
	}


	public static void ToLowerCase(char [] stringx){
		double i;

		for(i = 0d; i < stringx.Length; i = i + 1d){
			stringx[(int)(i)] = charToLowerCase(stringx[(int)(i)]);
		}
	}


	public static bool EqualsIgnoreCase(char [] a, char [] b){
		bool equal;
		double i;

		if(a.Length == b.Length){
			equal = true;
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(charToLowerCase(a[(int)(i)]) != charToLowerCase(b[(int)(i)])){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static char [] ReplaceString(char [] stringx, char [] toReplace, char [] replaceWith){
		char [] result;
		double i;
		BooleanReference equalsReference;
		bool success;

		equalsReference = new BooleanReference();
		result = new char [0];

		for(i = 0d; i < stringx.Length; ){
			success = SubstringEqualsWithCheck(stringx, i, toReplace, equalsReference);
			if(success){
				success = equalsReference.booleanValue;
			}

			if(success && toReplace.Length > 0d){
				result = ConcatenateString(result, replaceWith);
				i = i + toReplace.Length;
			}else{
				result = ConcatenateCharacter(result, stringx[(int)(i)]);
				i = i + 1d;
			}
		}

		return result;
	}


	public static char [] ReplaceCharacter(char [] stringx, char toReplace, char replaceWith){
		char [] result;
		double i;

		result = new char [0];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			if(stringx[(int)(i)] == toReplace){
				result = ConcatenateCharacter(result, replaceWith);
			}else{
				result = ConcatenateCharacter(result, stringx[(int)(i)]);
			}
		}

		return result;
	}


	public static char [] Trim(char [] stringx){
		char [] result;
		double i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
		bool firstNonWhitespaceFound;

		/* Find whitepaces at the start.*/
		lastWhitespaceLocationStart = -1d;
		firstNonWhitespaceFound = false;
		for(i = 0d; i < stringx.Length && !firstNonWhitespaceFound; i = i + 1d){
			if(charIsWhiteSpace(stringx[(int)(i)])){
				lastWhitespaceLocationStart = i;
			}else{
				firstNonWhitespaceFound = true;
			}
		}

		/* Find whitepaces at the end.*/
		lastWhitespaceLocationEnd = stringx.Length;
		firstNonWhitespaceFound = false;
		for(i = stringx.Length - 1d; i >= 0d && !firstNonWhitespaceFound; i = i - 1d){
			if(charIsWhiteSpace(stringx[(int)(i)])){
				lastWhitespaceLocationEnd = i;
			}else{
				firstNonWhitespaceFound = true;
			}
		}

		if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
			result = Substring(stringx, lastWhitespaceLocationStart + 1d, lastWhitespaceLocationEnd);
		}else{
			result = new char [0];
		}

		return result;
	}


	public static bool StartsWith(char [] stringx, char [] start){
		bool startsWithString;

		startsWithString = false;
		if(stringx.Length >= start.Length){
			startsWithString = SubstringEquals(stringx, 0d, start);
		}

		return startsWithString;
	}


	public static bool EndsWith(char [] stringx, char [] end){
		bool endsWithString;

		endsWithString = false;
		if(stringx.Length >= end.Length){
			endsWithString = SubstringEquals(stringx, stringx.Length - end.Length, end);
		}

		return endsWithString;
	}


	public static StringReference [] SplitByString(char [] toSplit, char [] splitBy){
		StringReference [] split;
		char [] next;
		double i;
		char c;
		StringReference n;

		split = new StringReference [0];

		next = new char [0];
		for(i = 0d; i < toSplit.Length; ){
			c = toSplit[(int)(i)];

			if(SubstringEquals(toSplit, i, splitBy)){
				if(split.Length != 0d || i != 0d){
					n = new StringReference();
					n.stringx = next;
					split = AddString(split, n);
					next = new char [0];
					i = i + splitBy.Length;
				}
			}else{
				next = AppendCharacter(next, c);
				i = i + 1d;
			}
		}

		if(next.Length > 0d){
			n = new StringReference();
			n.stringx = next;
			split = AddString(split, n);
		}

		return split;
	}


	public static bool StringIsBefore(char [] a, char [] b){
		bool before, equal, done;
		double i;

		before = false;
		equal = true;
		done = false;

		if(a.Length == 0d && b.Length > 0d){
			before = true;
		}else{
			for(i = 0d; i < a.Length && i < b.Length && !done; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
				if(charCharacterIsBefore(a[(int)(i)], b[(int)(i)])){
					before = true;
				}
				if(charCharacterIsBefore(b[(int)(i)], a[(int)(i)])){
					done = true;
				}
			}

			if(equal){
				if(a.Length < b.Length){
					before = true;
				}
			}
		}

		return before;
	}


	public static char charToLowerCase(char character){
		char toReturn;

		toReturn = character;
		if(character == 'A'){
			toReturn = 'a';
		}else if(character == 'B'){
			toReturn = 'b';
		}else if(character == 'C'){
			toReturn = 'c';
		}else if(character == 'D'){
			toReturn = 'd';
		}else if(character == 'E'){
			toReturn = 'e';
		}else if(character == 'F'){
			toReturn = 'f';
		}else if(character == 'G'){
			toReturn = 'g';
		}else if(character == 'H'){
			toReturn = 'h';
		}else if(character == 'I'){
			toReturn = 'i';
		}else if(character == 'J'){
			toReturn = 'j';
		}else if(character == 'K'){
			toReturn = 'k';
		}else if(character == 'L'){
			toReturn = 'l';
		}else if(character == 'M'){
			toReturn = 'm';
		}else if(character == 'N'){
			toReturn = 'n';
		}else if(character == 'O'){
			toReturn = 'o';
		}else if(character == 'P'){
			toReturn = 'p';
		}else if(character == 'Q'){
			toReturn = 'q';
		}else if(character == 'R'){
			toReturn = 'r';
		}else if(character == 'S'){
			toReturn = 's';
		}else if(character == 'T'){
			toReturn = 't';
		}else if(character == 'U'){
			toReturn = 'u';
		}else if(character == 'V'){
			toReturn = 'v';
		}else if(character == 'W'){
			toReturn = 'w';
		}else if(character == 'X'){
			toReturn = 'x';
		}else if(character == 'Y'){
			toReturn = 'y';
		}else if(character == 'Z'){
			toReturn = 'z';
		}

		return toReturn;
	}


	public static char charToUpperCase(char character){
		char toReturn;

		toReturn = character;
		if(character == 'a'){
			toReturn = 'A';
		}else if(character == 'b'){
			toReturn = 'B';
		}else if(character == 'c'){
			toReturn = 'C';
		}else if(character == 'd'){
			toReturn = 'D';
		}else if(character == 'e'){
			toReturn = 'E';
		}else if(character == 'f'){
			toReturn = 'F';
		}else if(character == 'g'){
			toReturn = 'G';
		}else if(character == 'h'){
			toReturn = 'H';
		}else if(character == 'i'){
			toReturn = 'I';
		}else if(character == 'j'){
			toReturn = 'J';
		}else if(character == 'k'){
			toReturn = 'K';
		}else if(character == 'l'){
			toReturn = 'L';
		}else if(character == 'm'){
			toReturn = 'M';
		}else if(character == 'n'){
			toReturn = 'N';
		}else if(character == 'o'){
			toReturn = 'O';
		}else if(character == 'p'){
			toReturn = 'P';
		}else if(character == 'q'){
			toReturn = 'Q';
		}else if(character == 'r'){
			toReturn = 'R';
		}else if(character == 's'){
			toReturn = 'S';
		}else if(character == 't'){
			toReturn = 'T';
		}else if(character == 'u'){
			toReturn = 'U';
		}else if(character == 'v'){
			toReturn = 'V';
		}else if(character == 'w'){
			toReturn = 'W';
		}else if(character == 'x'){
			toReturn = 'X';
		}else if(character == 'y'){
			toReturn = 'Y';
		}else if(character == 'z'){
			toReturn = 'Z';
		}

		return toReturn;
	}


	public static bool charIsUpperCase(char character){
		bool isUpper;

		isUpper = false;
		if(character == 'A'){
			isUpper = true;
		}else if(character == 'B'){
			isUpper = true;
		}else if(character == 'C'){
			isUpper = true;
		}else if(character == 'D'){
			isUpper = true;
		}else if(character == 'E'){
			isUpper = true;
		}else if(character == 'F'){
			isUpper = true;
		}else if(character == 'G'){
			isUpper = true;
		}else if(character == 'H'){
			isUpper = true;
		}else if(character == 'I'){
			isUpper = true;
		}else if(character == 'J'){
			isUpper = true;
		}else if(character == 'K'){
			isUpper = true;
		}else if(character == 'L'){
			isUpper = true;
		}else if(character == 'M'){
			isUpper = true;
		}else if(character == 'N'){
			isUpper = true;
		}else if(character == 'O'){
			isUpper = true;
		}else if(character == 'P'){
			isUpper = true;
		}else if(character == 'Q'){
			isUpper = true;
		}else if(character == 'R'){
			isUpper = true;
		}else if(character == 'S'){
			isUpper = true;
		}else if(character == 'T'){
			isUpper = true;
		}else if(character == 'U'){
			isUpper = true;
		}else if(character == 'V'){
			isUpper = true;
		}else if(character == 'W'){
			isUpper = true;
		}else if(character == 'X'){
			isUpper = true;
		}else if(character == 'Y'){
			isUpper = true;
		}else if(character == 'Z'){
			isUpper = true;
		}

		return isUpper;
	}


	public static bool charIsLowerCase(char character){
		bool isLower;

		isLower = false;
		if(character == 'a'){
			isLower = true;
		}else if(character == 'b'){
			isLower = true;
		}else if(character == 'c'){
			isLower = true;
		}else if(character == 'd'){
			isLower = true;
		}else if(character == 'e'){
			isLower = true;
		}else if(character == 'f'){
			isLower = true;
		}else if(character == 'g'){
			isLower = true;
		}else if(character == 'h'){
			isLower = true;
		}else if(character == 'i'){
			isLower = true;
		}else if(character == 'j'){
			isLower = true;
		}else if(character == 'k'){
			isLower = true;
		}else if(character == 'l'){
			isLower = true;
		}else if(character == 'm'){
			isLower = true;
		}else if(character == 'n'){
			isLower = true;
		}else if(character == 'o'){
			isLower = true;
		}else if(character == 'p'){
			isLower = true;
		}else if(character == 'q'){
			isLower = true;
		}else if(character == 'r'){
			isLower = true;
		}else if(character == 's'){
			isLower = true;
		}else if(character == 't'){
			isLower = true;
		}else if(character == 'u'){
			isLower = true;
		}else if(character == 'v'){
			isLower = true;
		}else if(character == 'w'){
			isLower = true;
		}else if(character == 'x'){
			isLower = true;
		}else if(character == 'y'){
			isLower = true;
		}else if(character == 'z'){
			isLower = true;
		}

		return isLower;
	}


	public static bool charIsLetter(char character){
		return charIsUpperCase(character) || charIsLowerCase(character);
	}


	public static bool charIsNumber(char character){
		bool isNumberx;

		isNumberx = false;
		if(character == '0'){
			isNumberx = true;
		}else if(character == '1'){
			isNumberx = true;
		}else if(character == '2'){
			isNumberx = true;
		}else if(character == '3'){
			isNumberx = true;
		}else if(character == '4'){
			isNumberx = true;
		}else if(character == '5'){
			isNumberx = true;
		}else if(character == '6'){
			isNumberx = true;
		}else if(character == '7'){
			isNumberx = true;
		}else if(character == '8'){
			isNumberx = true;
		}else if(character == '9'){
			isNumberx = true;
		}

		return isNumberx;
	}


	public static bool charIsWhiteSpace(char character){
		bool isWhiteSpacex;

		isWhiteSpacex = false;
		if(character == ' '){
			isWhiteSpacex = true;
		}else if(character == '\t'){
			isWhiteSpacex = true;
		}else if(character == '\n'){
			isWhiteSpacex = true;
		}else if(character == '\r'){
			isWhiteSpacex = true;
		}

		return isWhiteSpacex;
	}


	public static bool charIsSymbol(char character){
		bool isSymbolx;

		isSymbolx = false;
		if(character == '!'){
			isSymbolx = true;
		}else if(character == '\"'){
			isSymbolx = true;
		}else if(character == '#'){
			isSymbolx = true;
		}else if(character == '$'){
			isSymbolx = true;
		}else if(character == '%'){
			isSymbolx = true;
		}else if(character == '&'){
			isSymbolx = true;
		}else if(character == '\''){
			isSymbolx = true;
		}else if(character == '('){
			isSymbolx = true;
		}else if(character == ')'){
			isSymbolx = true;
		}else if(character == '*'){
			isSymbolx = true;
		}else if(character == '+'){
			isSymbolx = true;
		}else if(character == ','){
			isSymbolx = true;
		}else if(character == '-'){
			isSymbolx = true;
		}else if(character == '.'){
			isSymbolx = true;
		}else if(character == '/'){
			isSymbolx = true;
		}else if(character == ':'){
			isSymbolx = true;
		}else if(character == ';'){
			isSymbolx = true;
		}else if(character == '<'){
			isSymbolx = true;
		}else if(character == '='){
			isSymbolx = true;
		}else if(character == '>'){
			isSymbolx = true;
		}else if(character == '?'){
			isSymbolx = true;
		}else if(character == '@'){
			isSymbolx = true;
		}else if(character == '['){
			isSymbolx = true;
		}else if(character == '\\'){
			isSymbolx = true;
		}else if(character == ']'){
			isSymbolx = true;
		}else if(character == '^'){
			isSymbolx = true;
		}else if(character == '_'){
			isSymbolx = true;
		}else if(character == '`'){
			isSymbolx = true;
		}else if(character == '{'){
			isSymbolx = true;
		}else if(character == '|'){
			isSymbolx = true;
		}else if(character == '}'){
			isSymbolx = true;
		}else if(character == '~'){
			isSymbolx = true;
		}

		return isSymbolx;
	}


	public static bool charCharacterIsBefore(char a, char b){
		double ad, bd;

		ad = a;
		bd = b;

		return ad < bd;
	}


	public static void delete(System.Object objectx){
		// C# has garbage collection.
	}
}

}